package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AcceptanceRecordFormAuditDTO;
import digital.laboratory.platform.reagent.dto.AcceptanceRecordFormDTO;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.AcceptanceRecordFormVO;
import digital.laboratory.platform.reagent.vo.ProcurementContentVO;
import digital.laboratory.platform.reagent.vo.PurchasingPlanVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.time.LocalDate;
import java.util.List;

/**
 * (验收记录表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (验收记录表) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/acceptance_record_form")
@Api(value = "acceptance_record_form", tags = "(验收记录表)管理")
public class AcceptanceRecordFormController {

    private final AcceptanceSignForService acceptanceSignForService;

    private final AcceptanceRecordFormService acceptanceRecordFormService;

    private final ReagentConsumablesService reagentConsumablesService;

    private final OssFile ossFile;

    private final ReferenceMaterialService referenceMaterialService;

    private final BatchDetailsService batchDetailsService;

    private final WarehousingBatchListService warehousingBatchListService;

    private final WarehousingContentService warehousingContentService;

    /**
     * 通过id查询(验收记录表)
     *
     * @param acceptanceRecordFormId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping()
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_get')")
    public R<AcceptanceRecordFormVO> getById(String acceptanceRecordFormId, String rid, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        //通过标准物质ID查询该标准物质类的验收记录
        if (rid != null) {
            //只能从标准物质ID——》逐级查询出验收记录ID
            AcceptanceRecordFormVO acceptanceRecordFormVO =
                    acceptanceRecordFormService.getAcceptanceRecordFormVO
                            (warehousingContentService.getById
                                    (warehousingBatchListService.getById
                                            (batchDetailsService.getById
                                                    (referenceMaterialService.getById(rid)).getWarehousingBatchListId()).getWarehousingContentId()).getAcceptanceRecordFormId());
            if (acceptanceRecordFormVO.getStatus() == 6) {
                return R.ok(acceptanceRecordFormVO);
            } else return R.failed(null);

        }
        AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(acceptanceRecordFormId);
        return R.ok(acceptanceRecordFormVO);
    }

    /**
     * 分页查询
     *
     * @param page                  分页对象
     * @param reagentConsumableName (验收记录表)
     * @return
     */
    @ApiOperation(value = "分页查询验收任务", notes = "分页查询验收任务")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_page')")
    public R<IPage<AcceptanceRecordFormVO>> getAcceptanceRecordFormPage(Page<AcceptanceRecordForm> page, Integer status, String reagentConsumableName, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate endTime, HttpServletRequest theHttpServletRequest, String acceptanceSignForId) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
      /*查询参数
      * 标准物质名称
      * 验收时间
      * 验收状态
      * */
        IPage<AcceptanceRecordFormVO> acceptanceRecordFormVOPage = acceptanceRecordFormService.getAcceptanceRecordFormVOPage(page, Wrappers.<AcceptanceRecordForm>query()
                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .ge(startTime != null, "create_time", startTime)
                .le(endTime != null, "create_time", endTime)
                .eq(status != null, "status", status)
                .orderByDesc("create_time")
                .eq("acceptance_sign_for_id", acceptanceSignForId));
        return R.ok(acceptanceRecordFormVOPage);
    }


    /**
     * 录入(验收记录表)
     *
     * @param acceptanceRecordFormDTO (验收记录表)
     * @return R
     */
    @ApiOperation(value = "录入(验收记录表)", notes = "录入(验收记录表)")
    @SysLog("录入(验收记录表)")
    @PutMapping("/commit")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_add')")
    public R<AcceptanceRecordForm> postAddObject(@RequestBody AcceptanceRecordFormDTO acceptanceRecordFormDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        AcceptanceRecordForm acceptanceRecordForm = acceptanceRecordFormService.commitForm(acceptanceRecordFormDTO, dlpUser);
        if (acceptanceRecordForm != null) {
            return R.ok(acceptanceRecordForm, "保存成功");
        } else return R.failed("保存失败");

    }

    /**
     * 批量验收
     *
     * @param acceptanceRecordFormDTOList 批量验收
     * @return R
     */
    @ApiOperation(value = "批量验收(验收记录表)", notes = "批量验收(验收记录表)")
    @SysLog("批量验收")
    @PutMapping("/culkCommit")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_add')")
    public R<String> culkCommit(@RequestBody List<AcceptanceRecordFormDTO> acceptanceRecordFormDTOList, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        acceptanceRecordFormService.culkCommit(acceptanceRecordFormDTOList, dlpUser);
        return R.ok("批量验收成功");

    }

    /**
     * 一级审核(验收记录表)
     *
     * @param acceptanceRecordFormAuditDTO (验收记录表)
     * @return R
     */
    @ApiOperation(value = "一级审核(验收记录表)", notes = "一级审核(验收记录表)")
    @SysLog("一级审核(验收记录表)")
    @PutMapping("/primary")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_primary')")
    public R<String> primaryAudit(@RequestBody AcceptanceRecordFormAuditDTO acceptanceRecordFormAuditDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        if (acceptanceRecordFormService.primaryAudit(acceptanceRecordFormAuditDTO, dlpUser)) {
            return R.ok("审核成功");
        } else return R.failed("审核失败");
    }

    /**
     * 二级审核(验收记录表)
     *
     * @param acceptanceRecordFormAuditDTO (验收记录表)
     * @return R
     */
    @ApiOperation(value = "二级审核(验收记录表)", notes = "二级审核(验收记录表)")
    @SysLog("二级审核(验收记录表)")
    @PutMapping("/secondary")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_secondary')")
    public R<String> secondaryAudit(@RequestBody AcceptanceRecordFormAuditDTO acceptanceRecordFormAuditDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        if (acceptanceRecordFormService.secondaryAudit(acceptanceRecordFormAuditDTO, dlpUser)) {
            return R.ok("审核成功");
        } else return R.failed("审核失败");
    }

    /**
     * 三级审核(验收记录表)
     *
     * @param acceptanceRecordFormAuditDTO (验收记录表)
     * @return R
     */
    @ApiOperation(value = "三级审核(验收记录表)", notes = "三级审核(验收记录表)")
    @SysLog("三级审核(验收记录表)")
    @PutMapping("/three_level")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_threeLevel')")
    public R<String> threeLevelAudit(@RequestBody AcceptanceRecordFormAuditDTO acceptanceRecordFormAuditDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        if (acceptanceRecordFormService.threeLevelAudit(acceptanceRecordFormAuditDTO, dlpUser)) {
            return R.ok("审核成功");
        } else return R.failed("审核失败");
    }

    /**-------验收记录打印*/
    /**
     * 验收记录打印
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "验收记录打印", notes = "验收记录打印")
    @SysLog("验收记录打印")
    @PostMapping("/print")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_print')")
    public void bizGetPDFInventory(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) {
        System.out.println("bizApplyWord.................");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        AcceptanceRecordFormVO acceptanceRecordFormVO = acceptanceRecordFormService.getAcceptanceRecordFormVO(id);
        ReagentConsumables byId = reagentConsumablesService.getById(acceptanceRecordFormVO.getReagentConsumableId());

        String applyFileName = null;


        if (byId.getCategory().equals("试剂") | byId.getCategory().equals("耗材")) {

            applyFileName = "试剂耗材验收记录表" + id;
        } else {

            applyFileName = "标准物质验收记录表" + id;
        }


        String pdfFilePath = "document" + "/" + "acceptanceRecordForm" + "/" + id + "/" + applyFileName + ".pdf";
        try {
            //直接调用pdf方法
            acceptanceRecordFormService.acceptanceRecordFormTablePDF(acceptanceRecordFormVO, theHttpServletRequest, httpServletResponse);
            ossFile.fileGet(pdfFilePath, httpServletResponse.getOutputStream());
            httpServletResponse.setContentType(applyFileName);
        } catch (Exception e) {
            System.out.println(String.format("minioFile objectExist() Exception. %s", e.getLocalizedMessage()));
            e.printStackTrace();
        }
    }

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @ApiOperation(value = "分页查询验收集合主表", notes = "分页查询验收集合主表")
    @GetMapping("/acceptanceSignFor/Page")
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_recordPage')")
    public R<IPage<AcceptanceSignFor>> getacceptanceSignForPage(Page<AcceptanceSignFor> page, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate endTime, String acceptanceSignForId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        QueryWrapper<AcceptanceSignFor> queryWrapper = Wrappers.<AcceptanceSignFor>query().ge(startTime != null, "create_time", startTime)
                .le(endTime != null, "create_time", endTime)
                .orderByDesc("create_time");

        IPage<AcceptanceSignFor> page1 = acceptanceSignForService.page(page, queryWrapper);

        return R.ok(page1);
//		return R.ok(acceptanceRecordFormService.page(page, Wrappers.query(acceptanceRecordForm)));
    }

}
