package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.PurchaseListDTO;
import digital.laboratory.platform.reagent.entity.PurchaseList;
import digital.laboratory.platform.reagent.entity.PurchaseListDetails;
import digital.laboratory.platform.reagent.service.PurchaseListService;
import digital.laboratory.platform.reagent.vo.*;
import org.joda.time.DateTime;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.sql.Date;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * (采购清单)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购清单) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/purchase_list")
@Api(value = "purchase_list", tags = "(采购清单)管理")
public class PurchaseListController {

    private final PurchaseListService purchaseListService;
    private final OssFile ossFile;

    /**
     * 通过id查询(采购清单)
     *
     * @param purchaseListId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping()
    @PreAuthorize("@pms.hasPermission('reagent_purchase_list_get')")
    public R<PurchaseList> getById(String purchaseListId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseListVO purchaseList = purchaseListService.getPurchaseList(purchaseListId);

        return R.ok(purchaseList);
    }

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_list_page')")
    public R<IPage<PurchaseListPageVO>> getPurchaseListPage(Page<PurchaseList> page, Integer status, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") LocalDate endTime, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<PurchaseListPageVO> purchaseListSList = purchaseListService.getPurchaseListVOPage(page, Wrappers.<PurchaseList>query()
                .eq(status != null, "status", status)
                .ge(startTime != null, "create_time", startTime)
                .le(endTime != null, "create_time", endTime)
                .orderByDesc("create_time")
        );
        return R.ok(purchaseListSList);
//		return R.ok(purchaseListService.page(page, Wrappers.query(purchaseList)));
    }

    /**
     * 新增(采购清单)
     *
     * @param purchaseListDTO (采购清单)
     * @return R
     */
    @ApiOperation(value = "修改采购清单明细", notes = "修改采购清单明细")
    @SysLog("修改采购清单明细")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_purchase_list_add')")
    public R<String> postAddObject(@RequestBody List<PurchaseListDTO> purchaseListDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        purchaseListService.editById(purchaseListDTO);

        return R.ok("添加成功");
    }

    /**
     * 新增(采购清单)
     *
     * @param id (采购清单)
     * @return R
     */
    @ApiOperation(value = "提交采购清单明细", notes = "提交采购清单明细")
    @SysLog("提交采购清单明细")
    @PutMapping("/commit")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_list_commit')")
    public R<PurchaseList> commitById(String id, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseList purchaseList = purchaseListService.commitById(id);

        if (purchaseList != null) {
            return R.ok(purchaseList, "提交成功");
        } else {
            return R.failed(purchaseList, "提交失败");
        }
    }

}


