package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.ReagentConsumablesSet;
import digital.laboratory.platform.reagent.service.ReagentConsumablesSetService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (试剂耗材集合)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材集合) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/reagent_consumables_set" )
@Api(value = "reagent_consumables_set", tags = "(试剂耗材集合)管理")
public class ReagentConsumablesSetController {

    private final  ReagentConsumablesSetService reagentConsumablesSetService;

	/**
	 * 通过id查询(试剂耗材集合)
	 * @param reagentConsumablesSetid id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{reagentConsumablesSetid}" )
	@PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_set_get')" )
	public R<ReagentConsumablesSet> getById(@PathVariable("reagentConsumablesSetid" ) String reagentConsumablesSetid, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumablesSet reagentConsumablesSet = reagentConsumablesSetService.getById(reagentConsumablesSetid);
		return R.ok(reagentConsumablesSet);
		//return R.ok(reagentConsumablesSetService.getById(reagentConsumablesSetid));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param reagentConsumablesSet (试剂耗材集合)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_set_get')" )
    public R<IPage<ReagentConsumablesSet>> getReagentConsumablesSetPage(Page<ReagentConsumablesSet> page, ReagentConsumablesSet reagentConsumablesSet, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<ReagentConsumablesSet> reagentConsumablesSetSList = reagentConsumablesSetService.page(page, Wrappers.<ReagentConsumablesSet>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(reagentConsumablesSetSList);
//		return R.ok(reagentConsumablesSetService.page(page, Wrappers.query(reagentConsumablesSet)));
    }


    /**
     * 新增(试剂耗材集合)
     * @param reagentConsumablesSet (试剂耗材集合)
     * @return R
     */
    @ApiOperation(value = "新增(试剂耗材集合)", notes = "新增(试剂耗材集合)")
    @SysLog("新增(试剂耗材集合)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_set_add')" )
    public R<ReagentConsumablesSet> postAddObject(@RequestBody ReagentConsumablesSet reagentConsumablesSet, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        reagentConsumablesSet.setReagentConsumableId(IdWorker.get32UUID().toUpperCase());
		if (reagentConsumablesSetService.save(reagentConsumablesSet)) {
			return R.ok(reagentConsumablesSet, "对象创建成功");
		}
		else {
			return R.failed(reagentConsumablesSet, "对象创建失败");
		}
    }

    /**
     * 修改(试剂耗材集合)
     * @param reagentConsumablesSet (试剂耗材集合)
     * @return R
     */
    @ApiOperation(value = "修改(试剂耗材集合)", notes = "修改(试剂耗材集合)")
    @SysLog("修改(试剂耗材集合)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_set_edit')" )
    public R<ReagentConsumablesSet> putUpdateById(@RequestBody ReagentConsumablesSet reagentConsumablesSet, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (reagentConsumablesSetService.updateById(reagentConsumablesSet)) {
			return R.ok(reagentConsumablesSet, "保存对象成功");
		}
		else {
			return R.failed(reagentConsumablesSet, "保存对象失败");
		}
    }

    /**
     * 通过id删除(试剂耗材集合)
     * @param reagentConsumablesSetid id
     * @return R
     */
    @ApiOperation(value = "通过id删除(试剂耗材集合)", notes = "通过id删除(试剂耗材集合)")
    @SysLog("通过id删除(试剂耗材集合)" )
    @DeleteMapping("/{reagentConsumablesSetid}" )
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumables_set_del')" )
    public R<ReagentConsumablesSet> deleteById(@PathVariable String reagentConsumablesSetid, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumablesSet oldReagentConsumablesSet = reagentConsumablesSetService.getById(reagentConsumablesSetid);

		if (reagentConsumablesSetService.removeById(reagentConsumablesSetid)) {
			return R.ok(oldReagentConsumablesSet, "对象删除成功");
		}
		else {
			return R.failed(oldReagentConsumablesSet, "对象删除失败");
		}

    }

}
